#region Apache 2.0 License header

//   Copyright 2012 Lesley van Zijl
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
// http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License. 

#endregion

using System;
using System.Collections.Generic;
using System.Reflection;

namespace FavourObjects
{
    /// <summary>
    /// Container for properties of a Method for use in the MemberInfoCache.
    /// </summary>
    public class Method
    {
        /// <summary>
        /// Owner of the member.
        /// </summary>
        public Type Owner { get; set; }

        /// <summary>
        /// Parameters of the member.
        /// </summary>
        /// <remarks>This will be the FieldType if it's a field or the PropertyType if it's a property.</remarks>
        public IEnumerable<Type> Parameters { get; set; }

        /// <summary>
        /// The member's name.
        /// </summary>
        public String Name { get; set; }

        /// <summary>
        /// The MemberType of the member.
        /// </summary>
        public MemberTypes MemberType { get; set; }
    }
}