﻿#region Apache 2.0 License header

//   Copyright 2012 Lesley van Zijl
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
// http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License. 

#endregion

using System;
using System.Collections.Generic;
using System.Linq;
using System.Reflection;
using FavourObjects;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Tests.Helpers;

namespace Tests
{
    [TestClass]
    public class GenericClassTests
    {
        [TestMethod]
        public void Generic_int_Method_void()
        {
            var favourObject = FavourObject.Create<IGenericClass<int>>(new GenericTestClass<int>());

            bool b = favourObject.Method();

            Assert.IsTrue(b);
        }

        [TestMethod]
        public void Generic_CheckGenericClassType()
        {
            var favourObject = FavourObject.Create<IGenericClass<MethodInfo>>(new GenericTestClass<MethodInfo>());

            Type t = favourObject.GetGenericType();

            Assert.AreEqual(typeof (MethodInfo), t);
        }

        [TestMethod]
        public void Generics_CheckGenericMethod()
        {
            var favourObject = FavourObject.Create<IGenericClass<MethodInfo>>(new GenericTestClass<MethodInfo>());

            Type t = favourObject.GenericMethod<MethodInfo>();

            Assert.AreEqual(typeof (MethodInfo), t);
        }

        [TestMethod]
        public void Generics_CheckGenericMethod2()
        {
            var favourObject = FavourObject.Create<IGenericClass<MethodInfo>>(new GenericTestClass<MethodInfo>());

            IEnumerable<Type> t = favourObject.GenericMethod<MethodInfo, int>();

            Assert.AreEqual(typeof (MethodInfo), t.ElementAt(0));
            Assert.AreEqual(typeof (int), t.ElementAt(1));
        }

        [TestMethod]
        public void Generics_CleanMethod()
        {
            var favourObject = FavourObject.Create<IGenericClass<MethodInfo>>(new GenericTestClass<MethodInfo>());

            favourObject.MethodClean<int>();
        }

        #region Nested type: IGenericClass

        public interface IGenericClass<T>
        {
            bool Method();
            Type GetGenericType();
            Type GenericMethod<V>();
            IEnumerable<Type> GenericMethod<V, U>();
            void MethodClean<V>();
        }

        #endregion
    }
}